<?php
/* @wordpress-plugin
 * Plugin Name:       WooCommerce UPS Shipping Method Pro
 * Plugin URI:        https://wpruby.com/plugin/woocommerce-ups-shipping-method-pro/
 * Description:       WooCommerce UPS Shipping Method Pro
 * Version:           2.0.1
 * WC requires at least: 3.0
 * WC tested up to: 5.3
 * Author:            WPRuby
 * Author URI:        https://wpruby.com
 * Text Domain:       wpruby-ups-shipping-method
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Domain Path:       /languages
 */

namespace WPRuby_Ups;

use WPRuby_Ups\Core\WPRuby_UPS_Shipping_Configuration;
use WPRuby_Ups\Core\WPRuby_UPS_Shipping_Method;
use WPRuby_Ups\Includes\Ups_License_Handler;
use WPRuby_Ups\Includes\Ups_Updater;

define('UPS_LITE_URL', plugin_dir_url(__FILE__));
define('UPS_LITE_DIR', plugin_dir_path(__FILE__));
define('WPRUBY_UPS_CURRENT_VERSION', '2.0.1');
define('WPRUBY_UPS_STORE_URL', 'https://wpruby.com');
define('WPRUBY_UPS_ITEM_NAME', 'Woocommerce UPS Shipping Method Pro');


class WPRuby_UPS_Shipping_Pro {

	/**
	 * The single instance of the class.
	 *
	 * @var WPRuby_UPS_Shipping_Pro
	 * @since 1.0.0
	 */
	protected static $_instance = null;

	public static function get_instance(): WPRuby_UPS_Shipping_Pro
	{
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * WPRuby_UPS_Shipping_Lite constructor.
	 */
	public function __construct()
	{
		add_action('wp_loaded', [$this, 'license_handler']);
		add_filter('woocommerce_shipping_methods', [$this, 'add_ups_method']);
		add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), [$this, 'plugin_action_links'] );
	}

	/**
	 * @param $methods
	 *
	 * @return array
	 */
	public function add_ups_method( $methods )
	{
		$methods['ups'] = WPRuby_UPS_Shipping_Method::class;
		$methods['ups_config'] = WPRuby_UPS_Shipping_Configuration::class;
		return $methods;
	}

	public function plugin_action_links( $links ): array
	{
		$links[] = '<a href="https://wpruby.com/plugin/woocommerce-ups-shipping-method-pro/" target="_blank">Get the Pro version</a>';
		$links[] = '<a href="https://wpruby.com/submit-ticket/" target="_blank">Support</a>';
		return $links;
	}

	public function license_handler()
	{
		$license_handler = new Ups_License_Handler('wpruby_ups_license_key');
		$license_handler->setPluginName(WPRUBY_UPS_ITEM_NAME);

		// Update Handler
		new Ups_Updater(WPRUBY_UPS_STORE_URL, __FILE__, [
			'version' 	=> WPRUBY_UPS_CURRENT_VERSION,
			'license' 	=> trim(get_option('wpruby_ups_license_key')),
			'item_name'     => WPRUBY_UPS_ITEM_NAME,
			'author' 	=> 'Waseem Senjer',
			'url'           => home_url()
		]);
	}

}

require_once dirname(__FILE__ ) . '/vendor/autoload.php';
require_once dirname(__FILE__ ) . '/includes/autoloader.php';

/** initiate the plugin */
WPRuby_UPS_Shipping_Pro::get_instance();
